#include <Arduino.h>
#include <SoftwareSerial.h>
#include "mapping.h"
#include "protocoleBT.h"
#include "CUltrason.h"

#ifdef MODE_BLUETOOTH_SIMULE
#define BT  Serial
#else
extern SoftwareSerial BT;
#endif


CUltrason::CUltrason() {
  
}

void CUltrason::Setup(unsigned char pinTrig, unsigned char pinEcho) {
  ucPinTrig = pinTrig;
  ucPinEcho = pinEcho;
  bUltrasonON = false; // Ultrason désactivé par défaut
  // module ultrason HC-SR04
  pinMode(ucPinTrig, OUTPUT);
  pinMode(ucPinEcho, INPUT);
  digitalWrite(ucPinTrig, LOW); // Initialisation
  
}


// Fonctions relatives au capteur ultrason
int CUltrason::iMesureDistance(void) {
  digitalWrite(ucPinTrig, HIGH);                // Début de l'impulsion
  delayMicroseconds(10);                            // On attend 10 microsecondes
  digitalWrite(ucPinTrig, LOW);                 // On arrête l'impulsion
  int lecture_echo = pulseIn(ULTRASON_ECHO, HIGH);  // On lit la valeur
  // On calcule la distance en cm. S=2*cm/time, avec S=290 m/s à verifier
  int cm = lecture_echo / 58;                       
  return cm;
}

 
/*********************************************************************************************************************
 * void cmdActionnerUltrason(unsigned int msg)
 * 
 * L'argument msg est de la form XY (voir signification en tête de ce fichier)
 * 
 * On extrait le chiffre le plus à droite
 * 
 * Si ULTRASON_ON, on active la mesure de distance 
 * Si ULTRASON_OFF, on désactive la mesure de distance 
 * 
 ********************************************************************************************************************/
 int CUltrason::cmdActionnerUltrason(unsigned int msg) {
  int res = 0;
//  Serial.print("void cmdActionnerUltrason(unsigned int msg) - Message à traiter : ");
//  Serial.println(msg);
  unsigned char ucCommande = msg;
  unsigned int uiSousMessage = msg;
  if (msg >= PROTO_3) {
    ucCommande = msg / PROTO_3;
    uiSousMessage = msg - ucCommande * PROTO_3;
  }

//  Serial.print("void cmdActionnerUltrason(unsigned int msg) - Valeur : ");
//  Serial.println(ucCommande);
  unsigned long uiAEnvoyer=0; unsigned int ret = 0;
  switch(ucCommande) {
    case ULTRASON_ON : 
//      Serial.println("void cmdActionnerUltrason(unsigned int msg) - Activer la mesure de distance");
      res = cmdUltrasonOnOff(true, uiSousMessage);
      if (res == uiSousMessage) {
        // Envoyer acquittement
        uiAEnvoyer = CMD_INTERNE*PROTO_1 + ACTIONNER_ULTRASON*PROTO_2 + ULTRASON_ON*PROTO_3;
        //btRetourner(unsigned char c1, unsigned char c2, unsigned char c3);
//        Serial.println(uiAEnvoyer);
        BT.print(uiAEnvoyer);
      }
      break;
    case ULTRASON_OFF : 
//      Serial.println("void cmdActionnerUltrason(unsigned int msg) - désactiver la mesure de distance");
      res = cmdUltrasonOnOff(false, uiSousMessage);
      if (res == true) {
        // Envoyer acquittement
        uiAEnvoyer = CMD_INTERNE*PROTO_1 + ACTIONNER_ULTRASON*PROTO_2 + ULTRASON_OFF*PROTO_3;
        
//        Serial.println(uiAEnvoyer);
        BT.print(uiAEnvoyer);
      }
      break;
    case ULTRASON_UNE_MESURE : 
//      Serial.println("void cmdActionnerUltrason(unsigned int msg) - Effectuer une mesure de distance");
        ret = cmdUltrasonUneMesure(uiSousMessage);
//      if (ret == uiSousMessage) {
        // Envoyer acquittement
        //uiAEnvoyer = CMD_INTERNE*PROTO_1 + ACTIONNER_ULTRASON*PROTO_2 + ULTRASON_UNE_MESURE*PROTO_3 + ret;
        btRetourner(CMD_INTERNE, ACTIONNER_ULTRASON, ULTRASON_UNE_MESURE, ret);
//        Serial.println(uiAEnvoyer);
//        BT.print(uiAEnvoyer);
//      }
      break;
      
    default : 
      break;
  }  
  return msg;
 }
 
int CUltrason::cmdUltrasonOnOff(bool bAllumer, int iMsg) {
  bUltrasonON = bAllumer;
  return iMsg;
}

int CUltrason::cmdUltrasonUneMesure(int iMsg) {
  // On mesure la distance entre le robot et l'obstacle le plus proche
  int iDistance = 0;
  iDistance = iMesureDistance();  
  Serial.print("cmdUltrasonUneMesure() - Mesure de distance : "); Serial.print(iDistance); Serial.println(" cm");
  return iDistance;
}
